#ifndef _CCALL_H_
#define _CCALL_H_

#include "am.h"
#include "engine.h"
#include "heap.h"

extern STACK stack[];

extern HEAP *prepareHeapToBytes(HEAP *h, const char *t);

/*
 * Use these within the C routines to extract data from the RAW pointers
 * @TODO - strings will break as we use sized strings rather than terminated strings!
 */
extern void heapToBytes(int8 *bytes, HEAP *h);
extern HEAP *bytesToHeap(int8 *bytes);

#define TOCINT(h,t)    h[1].lv
#define TOCFLOAT(h,t)  h[1].fv
#define TOCSTRING(h,t) ((char*)(h+2))
#define TOCCHAR(h,t)   ((char)h[1].lv)
#define TOCBOOL(h,t)   ((boolean)h[1].lv)
#define TOCRAW(h,t)    prepareHeapToBytes(h,t)

#define FROMCINT(c,t)    stack[sp++].hp = mkInt(c)
#define FROMCFLOAT(c,t)  stack[sp++].hp = mkFloat(c)
#define FROMCSTRING(c,t) char *str = c; stack[sp++].hp = mkString((const char*)str, strlen(str))
#define FROMCCHAR(c,t)   stack[sp++].hp = mkChar(c)
#define FROMCBOOL(c,t)   stack[sp++].hp = mkBool(c)
#define FROMCRAW(c,t)    stack[sp++].hp = mkString(t,strlen(t)); stack[sp-1].hp = (HEAP*)c

#define CCALL0(hf,f, wr,tr)\
hf() { \
	wr(f(), tr); \
}

#define CCALL1(hf,f, w1,t1, wr,tr)\
hf() { \
	HEAP *arg1 = stack[--sp].hp; \
	wr(f( w1(arg1,t1)), tr); \
}

#define CCALL2(hf,f, w1,t1, w2,t2, wr,tr)\
hf() { \
	HEAP *arg1 = stack[--sp].hp; \
	HEAP *arg2 = stack[--sp].hp; \
	wr(f( w1(arg1,t1),  w2(arg2,t2)), tr); \
}

#define CCALL3(hf,f, w1,t1, w2,t2, w3,t3, wr,tr)\
hf() { \
	HEAP *arg1 = stack[--sp].hp; \
	HEAP *arg2 = stack[--sp].hp; \
	HEAP *arg3 = stack[--sp].hp; \
	wr(f( w1(arg1,t1), w2(arg2,t2), w3(arg3,t3)), tr); \
}

#define CCALL4(hf,f, w1,t1, w2,t2, w3,t3, w4,t4, wr,tr)\
hf() { \
	HEAP *arg1 = stack[--sp].hp; \
	HEAP *arg2 = stack[--sp].hp; \
	HEAP *arg3 = stack[--sp].hp; \
	HEAP *arg4 = stack[--sp].hp; \
	wr(f( w1(arg1,t1), w2(arg2,t2), w3(arg3,t3), w4(arg4,t4)), tr); \
}

#define CCALL5(hf,f, w1,t1, w2,t2, w3,t3, w4,t4, w5,t5, wr,tr)\
hf() { \
	HEAP *arg1 = stack[--sp].hp; \
	HEAP *arg2 = stack[--sp].hp; \
	HEAP *arg3 = stack[--sp].hp; \
	HEAP *arg4 = stack[--sp].hp; \
	HEAP *arg5 = stack[--sp].hp; \
	wr(f( w1(arg1,t1), w2(arg2,t2), w3(arg3,t3), w4(arg4,t4), w5(arg5,t5)), tr); \
}

#define CCALL6(hf,f, w1,t1, w2,t2, w3,t3, w4,t4, w5,t5, w6,t6, wr,tr)\
hf() { \
	HEAP *arg1 = stack[--sp].hp; \
	HEAP *arg2 = stack[--sp].hp; \
	HEAP *arg3 = stack[--sp].hp; \
	HEAP *arg4 = stack[--sp].hp; \
	HEAP *arg5 = stack[--sp].hp; \
	HEAP *arg6 = stack[--sp].hp; \
	wr(f( w1(arg1,t1), w2(arg2,t2), w3(arg3,t3), w4(arg4,t4), w5(arg5,t5), w6(arg6,t6)), tr); \
}
	   
#define CCALL7(hf,f, w1,t1, w2,t2, w3,t3, w4,t4, w5,t5, w6,t6, w7,t7, wr,tr)\
hf() { \
	HEAP *arg1 = stack[--sp].hp; \
	HEAP *arg2 = stack[--sp].hp; \
	HEAP *arg3 = stack[--sp].hp; \
	HEAP *arg4 = stack[--sp].hp; \
	HEAP *arg5 = stack[--sp].hp; \
	HEAP *arg6 = stack[--sp].hp; \
	HEAP *arg7 = stack[--sp].hp; \
	wr(f( w1(arg1,t1), w2(arg2,t2), w3(arg3,t3), w4(arg4,t4), w5(arg5,t5), w6(arg6,t6), w7(arg7,t7)), tr); \
}	   
/* 
 * Examples:
 *   CCALL(bob, TOCINT,"-",   TOCCHAR,"-", FROMCBOOL,"-")
 *   CCALL(bob, TOCRAW,"v4f", TOCCHAR,"-", FROMCRAW,"v3i")
 *
 */

#endif /*_CCALL_H_*/
