#ifndef _TEXTREAD_H_
#define _TEXTREAD_H_

#include "heap.h"

/* 
 * Type descriptor is of the form:
 * <type> := i | f | b | c         , where (i)nteger, (f)loat, (b)oolean, (c)haracter
 *         | v <size> <type>       , where (v)ector
 *         | t <size> <type>^size  , where (t)uple
 *         | s <size>              , where (s)tring of <size> chars
 *         | w <size>              , where (w)ord of <size> bytes with msb first
 *
 * <size> := {0..9}+
 * 
 * e.g. image = v320v200v3i
 * 
 *
 * Formatted input is:
 * <input> := <white>* <int> <white>    , where type = i
 *          | <white>* <float> <white>  , where type = f
 *          | <white>* <bool> <white>   , where type = b
 *          | .                         , where type = c
 *          | .^size                    , where type = s <size>
 *          | .^size                    , where type = w <size>
 *          | <input>^size              , where type = v|t <size> ...
 * 
 * <float> := [+|-] {0..9}+ ['.' {0..9}+] [ ('e'|'E') [+|-] {0..9}+ ]
 * <int>   := [+|-] {0..9}+
 * <bool>  := "true" | "false"
 */

struct textin_t; /* treat as opaque */
typedef struct textin_t textin_t;
struct textin_t {
	const char *type;
	char ch;
	const char *buff;
	int32 len; /* of buff, < 0 if problem: -1=eof, -2=formaterror */
	int32 pos;
	/* Additional support for file read */
	int fd; /* != -1 if file */
};

void setTextSourceString(textin_t *t, const char *type, const char *str, uint32 len);

void setTextSourceFile(textin_t *t, const char *type, int fd); 

boolean parseText(textin_t *t, HEAP *dest);

boolean isTextEOF(textin_t *t);
boolean isTextClosed(textin_t *t);

void closeText(textin_t *t); /* in the case of a file(except stdin) the file is closed */

#endif /*_TEXTREAD_H_*/
