#ifndef _parse_h_
#define _parse_h_
#include <stdio.h>
#include "ops.h"
#include "main.h"

typedef enum {
 ARG_STRING = 0, 
 ARG_CHAR, 
 ARG_INT, 
 ARG_BOOL,
 ARG_FLOAT
} operand_e;

struct operand_t;
typedef struct operand_t operand_t;

struct opcode_t;
typedef struct opcode_t opcode_t;

struct operand_t {
	operand_e type;
	operand_t *next;
	union {
		char *str; /* stored as string with escaped chars */
		char *ch;  /* stored as string with escaped char */
		int i;
		float f;
		boolean b;
	} value;
};

struct opcode_t {
	opcode_e type;
	opcode_t *next;
	operand_t *arg;
	int line; /* line number as occurs in the original Ham file */
};



/*
 * Parse 'file' to build opcodes into the 'instructions' list.
 * - All string are guaranteed to be shared.
 *
 */
void parse(FILE *file);
extern opcode_t *instructions;

boolean isPsuedo(opcode_e type);

/*
 * Either share the string or add to the internal string pool
 *
 */
char *findString(char *str); /* create a new shared string on failure */

/* for debug */
void dumpOpcodes(opcode_t *list, boolean linenos);
void dumpOpcode(char *buff, opcode_t *p);

/* util */
operand_t *operandAt(opcode_t *op, int pos);

#endif /* _parse_h_ */
